﻿class MMC1 : IMapper
{
    public string Name { get => "MMC1"; }
    public int Number { get => 1; }
    public int DefaultPrgSize { get => 256 * 1024; }
    public int DefaultChrSize { get => 128 * 1024; }

    void WriteMMC1(IFamicomDumperConnection dumper, ushort address, byte data)
    {
        var buffer = new byte[5];
        for (var i = 0; i < 5; i++)
        {
            buffer[i] = (byte)(data >> i);
        }
        dumper.WriteCpu(address, buffer);
    }

    public void DumpPrg(IFamicomDumperConnection dumper, List<byte> data, int size)
    {
        dumper.WriteCpu(0x8000, 0x80);
        WriteMMC1(dumper, 0x8000, 0b11100);

        var banks = size / 0x4000;

        for (var bank = 0; bank < banks; bank++)
        {
            Console.Write($"Reading PRG bank #{bank}/{banks}... ");
            WriteMMC1(dumper, 0xE000, (byte)bank);
            data.AddRange(dumper.ReadCpu(0x8000, 0x4000));
            Console.WriteLine("OK");
        }
    }

    public void DumpChr(IFamicomDumperConnection dumper, List<byte> data, int size)
    {
        dumper.WriteCpu(0x8000, 0x80);
        WriteMMC1(dumper, 0x8000, 0b11100);

        var banks = size / 0x1000;

        for (var bank = 0; bank < banks; bank++)
        {
            Console.Write($"Reading CHR bank #{bank}/{banks}... ");
            WriteMMC1(dumper, 0xA000, (byte)bank);
            data.AddRange(dumper.ReadPpu(0x0000, 0x1000));
            Console.WriteLine("OK");
        }
    }

    public void EnablePrgRam(IFamicomDumperConnection dumper)
    {
        dumper.WriteCpu(0x8000, 0x80);
        WriteMMC1(dumper, 0xE000, 0x00);
    }

    public MirroringType GetMirroring(IFamicomDumperConnection dumper)
    {
        return MirroringType.MapperControlled;
    }
}
